import os


def get_current_folder():
    return os.path.dirname(os.path.realpath(__file__))


def get_required_packages():
    curr_folder = get_current_folder()
    requirements_txt = os.path.join(curr_folder, 'requirements.txt')
    f = open(requirements_txt, 'r')
    install_requires = []
    for pkg in f.readlines():
        install_requires.append(pkg.strip())

    return install_requires




import datetime
from swiss_common_utils.utils.log.log_utils import get_logger, log_enter_and_exit, print_ascii_art


class RetentionService:
    logger = get_logger()

    def __init__(self, shelf_life_minutes=4320):
        self.shelf_life_minutes = shelf_life_minutes

    @log_enter_and_exit
    def run(self):
        print_ascii_art('RETENTION')
        deletion_time = datetime.datetime.now() - datetime.timedelta(minutes=self.shelf_life_minutes)

        self.logger.info('Deleting models built before: {}'.format(deletion_time.strftime('%Y-%m-%d %H:%M')))

        from django_model_builder_service.models import PredictionModel
        records_deleted, queryset = PredictionModel.objects.filter(build_time__lt=deletion_time).delete()
        self.logger.info('Deleted {} models from db'.format(records_deleted))



import os
import sys
import threading

from prometheus_client import Counter
from swiss_common_utils.utils.datetime.datetime_utils import time_string_to_seconds
from swiss_common_utils.utils.log.log_utils import get_logger, log_enter_and_exit
from swiss_common_utils.utils.yml.yaml_utils import yaml_to_flat_dict

from django_model_builder_service.django_model_builder_service import views
from django_model_builder_service.django_model_builder_service.builder.builder_service import BuilderService
from django_model_builder_service.django_model_builder_service.config.config_manager import ConfigManager
from django_model_builder_service.django_model_builder_service.metrics.metrics import *
from django_model_builder_service.django_model_builder_service.views import build_model, do_retention

logger = get_logger()


@log_enter_and_exit
def initialize_environment_variables_for_dev_environment():
    logger.info('[ENTER] initialize_environment_variables_for_dev_environment')
    if not is_debug():
        return

    config_path = 'service/config/config-dev.yml'
    var_dict = yaml_to_flat_dict(src_file=config_path)
    for k, v in var_dict.items():
        key = '_'.join(['CONFIG_SERVICE', k.upper()])
        logger.debug('setting environment variable: {}={}'.format(key, v))
        os.environ[key] = str(v)


def initialize_retention_service():
    retention_interval_str = views.config_manager.get_config_val_with_default('retention_interval',
                                                                              '1d')  # Default is 1d
    retention_interval_seconds = time_string_to_seconds(retention_interval_str)

    retention_shelf_life_str = views.config_manager.get_config_val_with_default('retention_shelf_life',
                                                                                '3d')  # Default is 3d
    retention_shelf_life_seconds = time_string_to_seconds(retention_shelf_life_str)
    retention_shelf_life_minutes = retention_shelf_life_seconds / 60

    logger.info(
        'Initializing retention. Interval: {}, Shelf life: {}'.format(retention_interval_str,
                                                                      retention_shelf_life_str))

    from django_model_builder_service.django_model_builder_service.retention.retention import RetentionService
    views.retention_service = RetentionService(shelf_life_minutes=retention_shelf_life_minutes)

    t = threading.Timer(retention_interval_seconds, do_retention)
    t.start()


def initialize_config_manager():
    views.config_manager = ConfigManager()


def initialize_builder_service(prediction_model_wrapper):
    views.builder_service = BuilderService(prediction_model_wrapper=prediction_model_wrapper,
                                           config_manager=views.config_manager)


def is_debug():
    try:
        return os.environ['CONFIG_SERVICE_SETTINGS_DEBUG'].lower() == 'true'
    except KeyError as e:
        logger.warning('No environment variable: CONFIG_SERVICE_SETTINGS_DEBUG. setting default value: True'.format())
        return True


def is_manage_command(command):
    args = sys.argv
    if 'manage.py' in args:
        if command in args:
            logger.info('Running manage.py runserver --> service init')
            return True
        else:
            logger.info('Running some manage.py command other than runserver --> no service init')
            return False

    return False


def should_start_initialize_sequence():
    if is_manage_command('runserver'):
        return True

    if not is_debug():
        logger.info('DEBUG=False --> service init')
        return True

    logger.info('should_start_initialize_sequence returned True')
    return False


def initialize_build_model_task():
    # Build first model (blocking)
    params = {}
    t = threading.Timer(0.0, build_model, args=[params])
    t.start()


def initialize_metrics_counters():
    metrics_dict[COUNTERS][SHOW_MODEL] = Counter(name='show_model', documentation='count calls for model show')
    metrics_dict[COUNTERS][GET_MODEL] = Counter(name='get_model', documentation='count calls for get model')


# Initialize all tasks here
@log_enter_and_exit
def initializae_background_tasks(prediction_model_wrapper):
    # migrate & test command may not finish due to the recurring task initialized next.
    # Initialize the recurring task only on runserver command
    if not should_start_initialize_sequence():
        return

    logger.info('Running initializing sequence for service app')

    # Initialization order matters - do not change!
    initialize_config_manager()
    prediction_model_wrapper.config = views.config_manager.get_config_dict()
    initialize_builder_service(prediction_model_wrapper=prediction_model_wrapper)
    initialize_metrics_counters()
    initialize_retention_service()
    initialize_build_model_task()



import time

import threading

from service.model.mrow_lock import mrow


def writer(L, value, after, rwlock, times):
    """Append value to L after a period of time."""
    try:
        lock = rwlock.writer()
        # Get another lock, to test the fact that obtaining multiple
        # write locks from the same thread context doesn't block (lock
        # reentrancy).
        lock2 = rwlock.writer()
        # Get a reader lock too; should be the same as getting another
        # writer since writers are inherently readers as well.
        lock3 = rwlock.reader()
        times.append(time.time())
        time.sleep(after)
        L.append(value)
    finally:
        times.append(time.time())
        lock3.release()
        lock2.release()
        lock.release()


def reader(L1, L2, after, rwlock, times):
    """Append values from L1 to L2 after a period of time."""
    try:
        lock = rwlock.reader()
        # Get another lock, to test the fact that obtaining multiple
        # write locks from the same thread context doesn't block (lock
        # reentrancy).
        lock2 = rwlock.reader()
        times.append(time.time())
        time.sleep(after)
        L2.extend(L1)
    finally:
        times.append(time.time())
        lock2.release()
        lock.release()


def readerTurnedWriter(L, value, after, rwlock, times):
    """Append value to L after a period of time."""
    try:
        lock = rwlock.reader()
        lock2 = rwlock.writer()
        times.append(time.time())
        time.sleep(after)
        L.append(value)
    finally:
        times.append(time.time())
        lock2.release()
        lock.release()


def test_reentrancy():
    lock = mrow.RWLock()
    # Reentrant read locks.
    rlock1 = lock.reader()
    rlock2 = lock.reader()
    rlock2.release()
    rlock1.release()
    # Reentrant write locks.
    wlock1 = lock.writer()
    wlock2 = lock.writer()
    wlock2.release()
    wlock1.release()
    # Writers are also readers.
    wlock = lock.writer()
    rlock = lock.reader()
    rlock.release()
    wlock.release()


def test_writeReadRead():
    lock = mrow.RWLock()
    W, R1, R2 = [], [], []
    TW, TR1, TR2 = [], [], []
    thread1 = threading.Thread(
        target=writer,
        args=(W, 'foo', 0.2, lock, TW),
    )
    thread2 = threading.Thread(
        target=reader,
        args=(W, R1, 0.2, lock, TR1),
    )
    thread3 = threading.Thread(
        target=reader,
        args=(W, R2, 0.2, lock, TR2),
    )
    thread1.start()
    time.sleep(0.1)
    thread2.start()
    thread3.start()
    time.sleep(0.8)
    assert 'foo' in R1
    assert 'foo' in R2
    assert TR1[0] <= TR2[1]  # Read 1 started during read 2.
    assert TR2[0] <= TR1[1]  # Read 2 started during read 1.
    assert TR1[0] >= TW[1]  # Read 1 started after write.
    assert TR2[0] >= TW[1]  # Read 2 started after write.


def test_writeReadReadWrite():
    lock = mrow.RWLock()
    W, R1, R2 = [], [], []
    TW1, TR1, TR2, TW2 = [], [], [], []
    thread1 = threading.Thread(
        target=writer,
        args=(W, 'foo', 0.3, lock, TW1),
    )
    thread2 = threading.Thread(
        target=reader,
        args=(W, R1, 0.3, lock, TR1),
    )
    thread3 = threading.Thread(
        target=reader,
        args=(W, R2, 0.3, lock, TR2),
    )
    thread4 = threading.Thread(
        target=writer,
        args=(W, 'bar', 0.3, lock, TW2),
    )
    thread1.start()
    time.sleep(0.1)
    thread2.start()
    time.sleep(0.1)
    thread3.start()
    time.sleep(0.1)
    thread4.start()
    time.sleep(1.7)
    assert 'foo' in R1
    assert 'foo' in R2
    assert 'bar' not in R1
    assert 'bar' not in R2
    assert 'bar' in W
    assert TR1[0] <= TR2[1]  # Read 1 started during read 2.
    assert TR2[0] <= TR1[1]  # Read 2 started during read 1.
    assert TR1[0] >= TW1[1]  # Read 1 started after write 1.
    assert TR2[0] >= TW1[1]  # Read 2 started after write 1.
    assert TW2[0] >= TR1[1]  # Write 2 started after read 1.
    assert TW2[0] >= TR2[1]  # Write 2 started after read 2.


def test_writeReadReadtowrite():
    lock = mrow.RWLock()
    W, R1 = [], []
    TW1, TR1, TW2 = [], [], []
    thread1 = threading.Thread(
        target=writer,
        args=(W, 'foo', 0.3, lock, TW1),
    )
    thread2 = threading.Thread(
        target=reader,
        args=(W, R1, 0.3, lock, TR1),
    )
    thread3 = threading.Thread(
        target=readerTurnedWriter,
        args=(W, 'bar', 0.3, lock, TW2),
    )
    thread1.start()
    time.sleep(0.1)
    thread2.start()
    time.sleep(0.1)
    thread3.start()
    time.sleep(1.7)
    assert 'foo' in R1
    assert 'bar' not in R1
    assert 'bar' in W
    assert TR1[0] >= TW1[1]  # Read 1 started after write 1.
    assert TW2[0] >= TR1[1]  # Write 2 started after read 1.


def main():
    test_reentrancy()
    test_writeReadRead()
    test_writeReadReadWrite()
    test_writeReadReadtowrite()


if __name__ == "__main__":
    main()


# source: http://code.activestate.com/recipes/413393-multiple-reader-one-writer-mrow-resource-locking/
import threading
from threading import get_ident


def acquireLocked(fn):
    def _acquireLocked_wrapper(self, *args, **kw):
        L = self.acquireLock
        try:
            L.acquire()
            result = fn(self, *args, **kw)
        finally:
            L.release()
        return result

    return _acquireLocked_wrapper


def releaseLocked(fn):
    def _releaseLocked_wrapper(self, *args, **kw):
        L = self.releaseLock
        try:
            L.acquire()
            result = fn(self, *args, **kw)
        finally:
            L.release()
        return result

    return _releaseLocked_wrapper


class RWLock(object):
    """MROW resource lock."""

    def __init__(self):
        self.acquireLock = threading.Lock()
        self.releaseLock = threading.Lock()
        self.sublocks = []
        self.waiting = []
        self.readers = 0
        self.writing = False
        self.threadReaders = {}
        self.threadWriters = {}

    def reader(self):
        """Return an acquired read lock."""
        threadReaders, threadWriters = self.threadReaders, self.threadWriters
        ident = get_ident()
        if ident in threadReaders:
            sublock, count = threadReaders[ident]
            threadReaders[ident] = (sublock, count + 1)
            return sublock
        elif ident in threadWriters:
            # Writers are inherently readers, so treat as a reentrant
            # write lock.
            sublock, count = threadWriters[ident]
            threadWriters[ident] = (sublock, count + 1)
            return sublock
        sublock = RLock(self)
        if self.writing:
            # Wait for acquired writers to release.
            self.waiting.append(sublock)
            sublock.acquire()
        sublock.acquire()
        self.readers += 1
        self.sublocks.append(sublock)
        threadReaders[ident] = (sublock, 1)
        return sublock

    reader = acquireLocked(reader)

    def writer(self):
        """Return an acquired write lock."""
        threadReaders, threadWriters = self.threadReaders, self.threadWriters
        ident = get_ident()
        wasReader = False
        if ident in threadWriters:
            sublock, count = threadWriters[ident]
            threadWriters[ident] = (sublock, count + 1)
            return sublock
        elif ident in threadReaders:
            # Readers-turned-writers must wait for any reads to complete
            # before turning into writers.
            sublock, count = threadReaders[ident]
            del threadReaders[ident]
            self.readers -= 1
            self.sublocks.remove(sublock)
            wasReader = True
        sublock = WLock(self)
        if self.readers or self.writing:
            # Wait for acquired readers/writers to release.
            self.waiting.append(sublock)
            sublock.acquire()
        sublock.acquire()
        self.writing = True
        self.sublocks.append(sublock)
        if not wasReader:
            count = 0
        threadWriters[ident] = (sublock, count + 1)
        return sublock

    writer = acquireLocked(writer)

    def _releaseR(self, sublock):
        sublocks = self.sublocks
        if sublock in sublocks:
            threadReaders = self.threadReaders
            ident = get_ident()
            count = threadReaders[ident][1] - 1
            if count:
                threadReaders[ident] = (sublock, count)
            else:
                del threadReaders[ident]
                self.readers -= 1
                sublocks.remove(sublock)
                waiting = self.waiting
                if waiting and not self.readers:
                    # If a lock is waiting at this point, it is a write lock.
                    waiting.pop(0)._release()

    _releaseR = releaseLocked(_releaseR)

    def _releaseW(self, sublock):
        sublocks = self.sublocks
        if sublock in sublocks:
            threadWriters = self.threadWriters
            ident = get_ident()
            count = threadWriters[ident][1] - 1
            if count:
                threadWriters[ident] = (sublock, count)
            else:
                del threadWriters[ident]
                self.writing = False
                sublocks.remove(sublock)
                waiting = self.waiting
                # Release any waiting read locks.
                while waiting and isinstance(waiting[0], RLock):
                    waiting.pop(0)._release()

    _releaseW = releaseLocked(_releaseW)


class SubLock(object):

    def __init__(self, rwlock):
        self.lock = threading.Lock()
        self.rwlock = rwlock

    def _release(self):
        self.lock.release()

    def acquire(self):
        self.lock.acquire()


class RLock(SubLock):

    def release(self):
        self.rwlock._releaseR(self)


class WLock(SubLock):

    def release(self):
        self.rwlock._releaseW(self)



from unittest import TestCase, mock

from django_model_builder_service.django_model_builder_service.builder.builder_service import BuilderService
from django_model_builder_service.django_model_builder_service.config.config_manager import ConfigManager


class TestBuilderService(TestCase):

    def test_model_data_verify_saving_old_model_verify_wrapper_build(self):
        builder_service = BuilderService(config_manager=ConfigManager())
        with mock.patch.object(builder_service, '_save_model_to_db') as mock_save_old_model:
            with mock.patch.object(builder_service.wrapper, 'build') as mock_wrapper_build:
                mock_wrapper_build.return_value = {
                    'model': b'test_model',
                    'identifier': 'test_identifier',
                    'build_time': 'test_build_time',
                    'build_duration': 777
                }
                builder_service.build_model()
                self.assertEquals(mock_save_old_model.call_count, 1)
                self.assertEqual(mock_wrapper_build.call_count, 1)



import os
from swiss_common_utils.utils.log.log_utils import get_logger


class ConfigManager:
    logger = get_logger()

    CONFIG_SERVICE_PREFIX = 'CONFIG_SERVICE'
    MODEL_CONFIG_PREFIX = 'MODEL_CONFIG_'

    def __init__(self, config_dict=None):
        if config_dict:
            self.__config_dict = config_dict
            return

        self.__config_dict = {}

        env_vars = os.environ
        for var in env_vars:
            if var.startswith(self.CONFIG_SERVICE_PREFIX):
                val = env_vars[var]

                key = var.replace(self.CONFIG_SERVICE_PREFIX + '_', '')
                if self.is_model_config_key(key):
                    key = key.replace(self.MODEL_CONFIG_PREFIX, '')
                    if 'model' not in self.__config_dict:
                        self.__config_dict['model'] = {}
                    self.__config_dict['model'][key.lower()] = val

                else:
                    self.__config_dict[key.lower()] = val

        self.logger.info('Config object: {}'.format(self.__config_dict))

    def is_model_config_key(self, key):
        return key.startswith(self.MODEL_CONFIG_PREFIX)

    def get_config_val(self, key):
        return self.__config_dict[key]

    def get_config_val_with_default(self, key, default):
        try:
            return self.get_config_val(key)
        except KeyError as e:
            self.logger.warning('No config key: {}. return default value: {}'.format(key, default))
            return default

    def is_config_key_exists(self, key):
        try:
            self.get_config_val(key)
            return True
        except KeyError as e:
            return False

    def get_config_dict(self):
        return self.__config_dict



import base64
import io
import pickle

from memory_profiler import profile
from swiss_common_utils.utils.datetime.datetime_utils import format_time_length
from swiss_common_utils.utils.log.log_utils import log_enter_and_exit, get_logger, print_ascii_art

from django_model_builder_service.django_model_builder_service.builder.mrow_lock.mrow import RWLock

memory_usage_stream_build_model = io.StringIO('')


class BuilderService:
    logger = get_logger()
    mrow_lock = RWLock()

    def lock(type):
        def wrap(org_func):
            def wrapped_func(self, *args, **kwargs):
                self.logger.debug('acquiring {} lock'.format(type))

                if type == 'writer':
                    lock = self.mrow_lock.writer()
                else:
                    lock = self.mrow_lock.reader()

                result = org_func(self, *args, **kwargs)

                lock.release()
                self.logger.debug('releasing {} lock'.format(type))

                return result

            return wrapped_func

        return wrap

    def __init__(self, prediction_model_wrapper, config_manager):
        self.model_data = None

        self.config_manager = config_manager

        config_dict = self.config_manager.get_config_dict()

        self.wrapper = prediction_model_wrapper

    @lock(type='reader')
    def __get_model_data(self):
        return self.model_data

    @lock(type='writer')
    def __set_model_data(self, model_data):
        self.model_data = model_data

    @log_enter_and_exit
    def _save_model_to_db(self):
        try:
            model_data = self.__get_model_data()

            if model_data:
                identifier = model_data['identifier']
                self.logger.info('saving model {} to db'.format(identifier))
                from django_model_builder_service.models import PredictionModel
                pickled_model = pickle.dumps(model_data['model'])
                global memory_usage_stream_build_model
                PredictionModel.objects.create(model=pickled_model,
                                               identifier=identifier,
                                               build_time=model_data['build_time'],
                                               stats=model_data['stats'],
                                               build_duration=model_data['build_duration'],
                                               config=self.config_manager.get_config_val_with_default('model', '{}'),
                                               memory_usage=memory_usage_stream_build_model.getvalue())
                self.logger.info('saving model {} to db - DONE'.format(identifier))

        except Exception as e:
            self.logger.error('Exception caught. type: {}, message: {}'.format(type(e), str(e)))

    def __encode_model(self, model_data):
        model = model_data['model']
        model_enc = base64.b64encode(model)
        model_data['model'] = model_enc.decode(
            'utf-8')  # Convert byte string to string (for json serialization to send over the network)
        return model_data

    @profile(stream=memory_usage_stream_build_model)
    def __build_model_with_memory_profiling(self):
        return self.wrapper.build()

    @log_enter_and_exit
    def build_model(self):
        print_ascii_art('BUILD MODEL')
        model_data = self.__build_model_with_memory_profiling()
        model_data = self.__encode_model(model_data)
        self.__set_model_data(model_data)
        self._save_model_to_db()
        self.logger.info('new model identifier: {}'.format(model_data['identifier']))
        self.logger.info('new model build time: {}'.format(model_data['build_time']))
        self.logger.info(
            'new model build duration: {}'.format(format_time_length(seconds=model_data['build_duration'])))
        global memory_usage_stream_build_model
        memory_usage_stream_build_model.seek(0)  # Set the file pointer back to 0 so old report is overwritten
        memory_usage_str = memory_usage_stream_build_model.getvalue()
        self.logger.info('Memory usage summary: {}'.format(memory_usage_str))

        return True

    @log_enter_and_exit
    def get_model_data(self):
        self.logger.info('Getting model...')
        model_data = self.__get_model_data()

        if model_data:
            self.logger.info('model identifier: {}'.format(model_data['identifier']))
            self.logger.info('model build time: {}'.format(model_data['build_time']))
            return model_data

        raise ModelNotBuildError('no model was build yet')



from django.urls import path

from django_model_builder_service.django_model_builder_service import views

app_name = 'service'

urlpatterns = [
    path('getmodel/<str:identifier>/', views.show_model, name='show_model'),
    path('getmodel/', views.get_model, name='get_model'),
]



import json
from django.http import HttpRequest
from unittest import TestCase

from django_model_builder_service.django_model_builder_service.tests.test_utils import response_json_to_dict
from django_model_builder_service.django_model_builder_service.views import get_model


class TestApi(TestCase):

    def test_get_model_with_post_request(self):
        request = HttpRequest()
        request.method = 'POST'
        request.content_type = 'application/json'
        request.POST = json.loads('{}')
        response = get_model(request)
        response_as_dict = response_json_to_dict(response)
        self.assertEqual(response_as_dict, {'error': 'not a get request'})



import json
import threading

import pytz
from django.http import JsonResponse
# Create your views here.
from django.shortcuts import render, get_object_or_404
from django.utils import timezone
from swiss_common_utils.utils.datetime.datetime_utils import time_string_to_seconds, format_time_length
from swiss_common_utils.utils.log.log_utils import get_logger, log_enter_and_exit

from django_model_builder_service.django_model_builder_service.metrics.metrics import *

config_manager = None
builder_service = None
retention_service = None

logger = get_logger()


def generate_error_json_response(error):
    return JsonResponse({"error": error})


def generate_exception_error_message(e):
    return 'Exception caught. type: {}, message: {}'.format(type(e), str(e))


@log_enter_and_exit
def get_model(request):
    # metrics_dict[COUNTERS][GET_MODEL].inc()
    if request.method != 'GET':
        return generate_error_json_response('not a get request')

    try:
        global builder_service
        latest_model = builder_service.get_model_data()
        logger.info('Size of response: {} KB'.format(int(len(str(latest_model).encode('utf-8')) / 1024)))
        return JsonResponse(latest_model)
    except Exception as e:
        error_msg = generate_exception_error_message(e)
        logger.error(error_msg)
        return generate_error_json_response(error_msg)


@log_enter_and_exit
def build_model(params):
    try:
        global builder_service
        params['result'] = builder_service.build_model()

        global config_manager
        build_interval_seconds_str = config_manager.get_config_val_with_default('model_build_interval', '0.0s')
        build_interval_seconds = time_string_to_seconds(build_interval_seconds_str)
        logger.info('Model build interval: {}'.format(build_interval_seconds_str))
        if build_interval_seconds > 10.0:
            temp_params = {}
            t = threading.Timer(function=build_model, args=[temp_params], interval=build_interval_seconds)
            t.start()

    except Exception as e:
        msg = generate_exception_error_message(e)
        logger.error(msg)
        params['result'] = msg


@log_enter_and_exit
def do_retention():
    global retention_service
    retention_service.run()

    global config_manager
    retention_interval_str = config_manager.get_config_val_with_default('retention_interval',
                                                                        '1d')  # Default 1d
    retention_interval_seconds = time_string_to_seconds(retention_interval_str)
    logger.info('Model retention interval: {}'.format(retention_interval_str))
    if retention_interval_seconds > 10.0:
        t = threading.Timer(function=build_model, interval=retention_interval_seconds)
        t.start()


def show_model(request, identifier):
    # metrics_dict[COUNTERS][SHOW_MODEL].inc()

    from django_model_builder_service.django_model_builder_service.models import PredictionModel
    model = get_object_or_404(PredictionModel, identifier=identifier)
    try:
        stats = json.loads(model.stats.replace("'", '"'))
        config = json.loads(model.config.replace("'", '"'))
    except Exception as e:
        logger.warning(generate_exception_error_message(e))
        stats = model.stats
        config = model.config

    my_tz = pytz.timezone('Israel')
    timezone.activate(my_tz)
    now = timezone.localtime()
    build_time = timezone.localtime(model.build_time)
    context = {
        'now': now.strftime('%d %B %Y, %H:%M:%S'),
        'identifier': identifier,
        'build_time': build_time.strftime('%d %B %Y, %H:%M:%S'),
        'stats': stats,
        'build_duration': format_time_length(seconds=model.build_duration),
        'config': config,
        'memory_usage': model.memory_usage
    }

    return render(request, 'service/model.html', context)



from setuptools import setup, find_packages

from install_requirements import get_required_packages
from version import get_package_version

setup(
    name='django-model-builder-service',
    version=get_package_version(),
    packages=find_packages(),
    include_package_data=True,
    url='',
    license='BSD License',
    author='lior.yehonatan',
    author_email='lior.yehonatan@inel.com',
    description='Machine learning model builder app',
    python_requires='>=3.6',
    install_requires=get_required_packages()
)




from django.db import models


# Create your models here.


class PredictionModel(models.Model):
    model = models.BinaryField()
    identifier = models.CharField(max_length=32)
    build_time = models.DateTimeField('build date')
    stats = models.TextField(name='stats', default='{}')
    build_duration = models.IntegerField(name='build_duration', default=0)
    config = models.TextField(name='config', default={})
    memory_usage = models.TextField(name='memory_usage', default='')

    def __str__(self):
        return '[{}] {}'.format(self.build_time, self.identifier)
